@@ -1,4 +1,4 @@
-from .component import Component
+from .bipole import Component
 
 
 class Port(Component):
@@ -6,3 +6,4 @@ class Port(Component):
     type = "P"
     args = ()
     sketch_net = 'P 1 2'
+    has_value = False
