@@ -61,7 +61,7 @@ class Sketch:
             sketch_path = sketch_path.transform(Affine2D(svga_path.transform))
             sketch_paths.append(sketch_path)
 
-        sketch = cls(sketch_paths, svg.width, svg.height).align()
+        sketch = cls(sketch_paths, svg.width, svg.height).align(sketch_key)
         return sketch
 
     @classmethod
@@ -83,19 +83,37 @@ class Sketch:
         a.draw(str(svg_filename), label_values=False, label_ids=False,
                label_nodes=False, draw_nodes=False)
 
-    def offsets(self):
+    def offsets1(self, sketch_key):
+
+        if sketch_key == 'opamp':
+            return 0, self.height / 2
+
+        # TODO, use sketch_key to help find offset.
 
         xoffset = None
         yoffset = None
 
         # Look for pair of horizontal wires
+        candidates = []
         for path in self.paths:
-            if len(path.path) == 4 and all(path.path.codes == (1, 2, 1, 2)):
+            if len(path.path) >= 4 and all(path.path.codes[0:4] == (1, 2, 1, 2)):
                 vertices = path.path.vertices
                 if vertices[0][1] == vertices[1][1]:
                     xoffset = vertices[0][0]
                     yoffset = vertices[0][1]
-                    return 0, yoffset
+                    candidates.append((xoffset, yoffset))
+
+        if candidates != []:
+            # Search for horizontal line with longest extent.
+            # This is needed for fet/bjt.
+            xmin = 1000
+            yoffset = 0
+            for candidate in candidates:
+                if candidate[0] < xmin:
+                    xmin = candidate[0]
+                    yoffset = candidate[1]
+
+            return 0, yoffset
 
         # Look for vertical wire (for ground, sground, cground, rground)
         # Note, if look for horizontal wire first, get incorrect offset for rground
@@ -118,10 +136,15 @@ class Sketch:
 
         return self.width / 2, self.height / 2
 
-    def align(self):
+    def offsets(self, sketch_key):
+
+        xoffset, yoffset = self.offsets1(sketch_key)
+        return xoffset, yoffset
+
+    def align(self, sketch_key):
         """Remove yoffset from component."""
 
-        xoffset, yoffset = self.offsets()
+        xoffset, yoffset = self.offsets(sketch_key)
 
         if xoffset is None:
             return self
