@@ -1,14 +1,17 @@
 from ..annotation import Annotation
 from ..annotations import Annotations
 from .preferences import Preferences
-from ..components.cpt_maker import cpt_make, cpt_remake
+from ..components.opamp import Opamp
+from ..components.cpt_maker import cpt_make_from_cpt, cpt_make_from_type, cpt_remake
 
 from copy import copy
 from math import atan2, degrees, sqrt
-from lcapy import Circuit
-from lcapy.mnacpts import Cpt, Eopamp
+from numpy import nan, isnan
+from lcapy import Circuit, expr
+from lcapy.mnacpts import Cpt
 from lcapy.nodes import parse_nodes
 from lcapy.schemmisc import Pos
+from lcapy.opts import Opts
 
 
 class UIModelBase:
@@ -17,37 +20,46 @@ class UIModelBase:
     SNAP = 1
     SCALE = 0.25
 
+    # Short-cut key, name, type, kind
     component_map = {
-        'c': ('Capacitor', 'C', ''),
-        'd': ('Diode', 'D', ''),
-        'i': ('Current source', 'I', ''),
-        'l': ('Inductor', 'L', ''),
-        'r': ('Resistor', 'R', ''),
-        'nr': ('Noiseless resistor', 'R', ''),
-        'v': ('Voltage source', 'V', ''),
-        'w': ('Wire', 'W', ''),
-        'e': ('VCVS', 'E', ''),
-        'f': ('CCCS', 'F', ''),
-        'g': ('VCCS', 'G', ''),
-        'h': ('CCVS', 'H', ''),
-        'opamp': ('Opamp', 'Opamp', ''),
-        'p': ('Port', 'P', ''),
-        'y': ('Admittance', 'Y', ''),
-        'z': ('Impedance', 'Z', ''),
+        'c': ('c', 'Capacitor', 'C', ''),
+        'd': ('d', 'Diode', 'D', ''),
+        'fb': ('', 'Ferrite bead', 'FB', ''),
+        'i': ('i', 'Current source', 'I', ''),
+        'j': ('j', 'JFET', 'J', ''),
+        'l': ('l', 'Inductor', 'L', ''),
+        'm': ('m', 'MOSFET', 'M', ''),
+        'q': ('q', 'BJT', 'Q', ''),
+        'r': ('r', 'Resistor', 'R', ''),
+        'nr': ('', 'Noiseless resistor', 'R', ''),
+        'v': ('v', 'Voltage source', 'V', ''),
+        'w': ('w', 'Wire', 'W', ''),
+        'e': ('e', 'VCVS', 'E', ''),
+        'f': ('f', 'CCCS', 'F', ''),
+        'g': ('g', 'VCCS', 'G', ''),
+        'h': ('h', 'CCVS', 'H', ''),
+        'o': ('o', 'Open circuit', 'O', ''),
+        'opamp': ('', 'Opamp', 'opamp', ''),
+        'p': ('p', 'Port', 'P', ''),
+        'y': ('y', 'Admittance', 'Y', ''),
+        'z': ('z', 'Impedance', 'Z', ''),
+        'cpe': ('', 'CPE', 'CPE', ''),
     }
 
     connection_map = {
-        '0': ('0V', 'Ground', ''),
-        '0V': ('0V', 'Ground', '0V'),
-        'ground': ('Ground', 'Ground', ''),
-        'sground': ('Signal ground', 'Ground', 'sground'),
-        'rground': ('Rail ground', 'Ground', 'rground'),
-        'cground': ('Chassis ground', 'Ground', 'cground'),
-        # 'vdd': ('VDD', 'A', 'vdd'),
-        # 'vss': ('VSS', 'A', 'vss'),
-        # 'input': ('Input', 'A', 'input'),
-        # 'output': ('Output', 'A', 'output'),
-        # 'bidir': ('Bidirectional', 'A', 'bidir')
+        '0': ('0', '0V', 'W', ''),
+        '0V': ('', '0V', 'W', '0V'),
+        'ground': ('', 'Ground', 'W', ''),
+        'sground': ('', 'Signal ground', 'W', 'sground'),
+        'rground': ('', 'Rail ground', 'W', 'rground'),
+        'cground': ('', 'Chassis ground', 'W', 'cground'),
+        'vdd': ('', 'VDD', 'W', 'vdd'),
+        'vss': ('', 'VSS', 'W', 'vss'),
+        'vcc': ('', 'VCC', 'W', 'vcc'),
+        'vee': ('', 'VEE', 'W', 'vee'),
+        'input': ('', 'Input', 'W', 'input'),
+        'output': ('', 'Output', 'W', 'output'),
+        'bidir': ('', 'Bidirectional', 'W', 'bidir')
     }
 
     def __init__(self, ui):
@@ -55,7 +67,7 @@ class UIModelBase:
         self.circuit = Circuit()
         self.ui = ui
         self._analysis_circuit = None
-        self.filename = ''
+        self.pathname = ''
         self.voltage_annotations = Annotations()
         self.selected = None
         self.last_expr = None
@@ -115,6 +127,9 @@ class UIModelBase:
 
     def choose_cpt_name(self, cpt_type):
 
+        if cpt_type == 'opamp':
+            cpt_type = 'E'
+
         num = 1
         while True:
             name = cpt_type + str(num)
@@ -122,27 +137,18 @@ class UIModelBase:
                 return name
             num += 1
 
-    def choose_node_name(self, nodes):
-
-        num = 1
-        while True:
-            name = str(num)
-            if name not in nodes:
-                return name
-            num += 1
-
     def con_create(self, con_key, x1, y1, x2, y2):
         """Create a new connection."""
 
         try:
-            cpt_type = self.connection_map[con_key][1]
+            cpt_type = self.connection_map[con_key][2]
         except KeyError:
             return None
 
         if cpt_type == '':
             return None
 
-        return self.thing_create(cpt_type, x1, y1, x2, y2)
+        return self.thing_create(cpt_type, x1, y1, x2, y2, kind='-' + con_key)
 
     def copy(self, cpt):
 
@@ -162,7 +168,7 @@ class UIModelBase:
             return
 
         try:
-            cpt_type = self.component_map[cpt_key][1]
+            cpt_type = self.component_map[cpt_key][2]
         except KeyError:
             return None
 
@@ -184,6 +190,7 @@ class UIModelBase:
             pass
 
         self.circuit.remove(cpt.name)
+        self.invalidate()
 
         if redraw:
             self.ui.clear()
@@ -195,7 +202,7 @@ class UIModelBase:
         if gcpt is None:
             return
 
-        gcpt.draw(self, self.ui.layer)
+        gcpt.draw(self, self.ui.sketcher)
 
         label_cpts = self.preferences.label_cpts
 
@@ -205,23 +212,30 @@ class UIModelBase:
         name = cpt.name
 
         try:
-            value = cpt.args[0]
+            if cpt.type in ('F', 'H'):
+                value = cpt.args[1]
+            elif cpt.type in ('P', ):
+                value = None
+            else:
+                value = cpt.args[0]
         except IndexError:
             value = None
 
         if value is None:
             value = ''
+            value_latex = ''
+        else:
+            value_latex = '$' + expr(value).latex() + '$'
 
+        label = ''
         if label_cpts == 'name+value':
-            if name != value:
-                label = name + '=' + value
+            if name != value and gcpt.has_value:
+                label = name + '=' + value_latex
             else:
                 label = name
         elif label_cpts == 'value':
             if value != '':
-                label = value
-            else:
-                label = name
+                label = value_latex
         elif label_cpts == 'name':
             label = name
         elif label_cpts == 'none':
@@ -237,7 +251,7 @@ class UIModelBase:
 
         draw_nodes = self.preferences.draw_nodes
         if draw_nodes != 'none':
-            for node in cpt.nodes:
+            for node in gcpt.drawn_nodes:
                 if node.port:
                     self.node_draw(node)
                     continue
@@ -250,34 +264,75 @@ class UIModelBase:
 
         label_nodes = self.preferences.label_nodes
         if label_nodes != 'none':
-            for node in cpt.nodes:
+            for node in gcpt.labelled_nodes:
+                if node.name[0] == '_':
+                    continue
 
                 if label_nodes == 'alpha' and not node.name[0].isalpha():
                     continue
 
                 x, y = node.pos.x, node.pos.y
-                x += 0.3
-                y += 0.3
+                # Should be x -= 0.1 but need to right justify.
+                x += 0.1
+                y += 0.1
                 ann = Annotation(self.ui, x, y, node.name)
                 ann.draw(fontsize=18)
                 gcpt.annotations.append(ann)
 
-    def cpt_find(self, n1, n2):
+    def cpt_find(self, node_name1, node_name2):
 
-        cpt2 = None
+        fcpt = None
         for cpt in self.circuit:
-            if (cpt.nodes[0].name == n1 and cpt.nodes[1].name == n2):
-                cpt2 = cpt
+            if (cpt.nodes[0].name == node_name1 and cpt.nodes[1].name == node_name2):
+                fcpt = cpt
                 break
-        if cpt2 is None:
+        if fcpt is None:
             self.exception(
-                'Cannot find a component with nodes %s and %s' % (n1, n2))
-        return cpt2
+                'Cannot find a component with nodes %s and %s' % (node_name1, node_name2))
+        return fcpt
 
     def cpt_remake(self, cpt):
 
-        newcpt = cpt._change_kind(cpt.gcpt.kind)
-        newcpt.gcpt = cpt.gcpt
+        gcpt = cpt.gcpt
+
+        if cpt.is_dependent_source and gcpt.type != 'Eopamp':
+            try:
+                newcpt = cpt._change_control(gcpt.control)
+            except Exception:
+                self.exception('Control component %s for %s deleted' %
+                               (gcpt.control, cpt.name))
+                return
+        elif gcpt.cpt_kind == cpt._kind:
+            newcpt = cpt
+        elif gcpt.type != 'Eopamp':
+            try:
+                newcpt = cpt._change_kind(gcpt.cpt_kind)
+            except Exception:
+                self.exception('Cannot change kind for %s' % cpt.name)
+                return
+        else:
+            newcpt = cpt
+
+        if gcpt.name != cpt.name:
+            try:
+                newcpt = newcpt._change_name(gcpt.name)
+            except Exception:
+                self.exception('Cannot change name for %s' % cpt.name)
+                return
+
+        if gcpt.mirror ^ ('mirror' in newcpt.opts):
+            # TODO, add mirror method...
+            if gcpt.type == 'Eopamp':
+                newcpt.nodes[2], newcpt.nodes[3] = newcpt.nodes[3], newcpt.nodes[2]
+            elif gcpt.type in ('J', 'M', 'Q'):
+                newcpt.nodes[2], newcpt.nodes[0] = newcpt.nodes[0], newcpt.nodes[2]
+            else:
+                print('Trying to change mirror for ' + str(newcpt))
+
+        newcpt.opts.clear()
+        newcpt.opts.add(gcpt.attr_string_update(self.STEP))
+
+        newcpt.gcpt = gcpt
         cpt_remake(newcpt.gcpt)
 
     def cut(self, cpt):
@@ -296,44 +351,77 @@ class UIModelBase:
             return
         cpt.draw(**kwargs)
 
-    def export(self, filename):
+    def export(self, pathname):
 
         cct = self.circuit
-        cct.draw(filename)
+        cct.draw(pathname)
 
     def invalidate(self):
 
         self._analysis_circuit = None
 
-    def load(self, filename):
+    def load(self, pathname):
 
         from lcapy import Circuit
 
-        self.filename = filename
+        self.pathname = pathname
 
-        with open(filename) as f:
+        with open(pathname) as f:
             line = f.readline()
             if line.startswith(r'\begin{tikz'):
                 self.ui.show_error_dialog('Cannot load Circuitikz macro file')
                 return
 
         try:
-            self.circuit = Circuit(filename)
+            circuit = Circuit(pathname)
         except Exception as e:
             self.exception(e)
             return
 
+        return self.load_from_circuit(circuit)
+
+    def load_from_circuit(self, circuit):
+
+        self.circuit = circuit
         positions = None
         for cpt in self.circuit.elements.values():
             if cpt.type == 'XX' and 'nodes' in cpt.opts:
                 positions = parse_nodes(cpt.opts['nodes'])
+                break
 
-        if positions is None:
-            self.exception('Node positions not defined')
-            return
+        if positions is not None:
+            for k, v in self.circuit.nodes.items():
+                try:
+                    v.pos = positions[k]
+                except KeyError:
+                    v.pos = None
 
-        for k, v in self.circuit.nodes.items():
-            v.pos = positions[k]
+        else:
+
+            # Node positions not defined.
+
+            sch = self.circuit.sch
+
+            try:
+                # This will fail if have detached components.
+                calculated = sch._positions_calculate()
+            except (AttributeError, ValueError, RuntimeError) as e:
+                self.exception(e)
+                return
+
+            width, height = sch.width * self.STEP, sch.height * self.STEP
+
+            # Centre the schematic.
+            xsize = self.ui.canvas.drawing.xsize
+            ysize = self.ui.canvas.drawing.ysize
+            offsetx, offsety = self.snap((xsize - width) / 2,
+                                         (ysize - height) / 2)
+            for node in sch.nodes.values():
+                node.pos.x += offsetx
+                node.pos.y += offsety
+                # May have split nodes...
+                if node.name in circuit.nodes:
+                    circuit.nodes[node.name].pos = node.pos
 
         self.remove_directives()
 
@@ -342,15 +430,9 @@ class UIModelBase:
                 cpt.gcpt = None
                 continue
             try:
-                cpt_type = cpt.type
-                if isinstance(cpt, Eopamp):
-                    cpt_type = 'Opamp'
-                gcpt = cpt_make(cpt_type, kind=cpt._kind)
-                # FIXME.
-                gcpt.name = cpt.name
-                gcpt.nodes = cpt.nodes
+                gcpt = cpt_make_from_cpt(cpt)
             except Exception as e:
-                cgpt = None
+                gcpt = None
                 self.exception(e)
 
             cpt.gcpt = gcpt
@@ -369,6 +451,12 @@ class UIModelBase:
 
         return self.thing_create(self.clipboard.type, x1, y1, x2, y2)
 
+    def possible_control_names(self):
+
+        cpts = self.circuit.elements.values()
+        names = [c.name for c in cpts if c.name[0] != 'W']
+        return names
+
     def remove_directives(self):
 
         elt_list = list(self.circuit.elements.values())
@@ -396,11 +484,11 @@ class UIModelBase:
         # TODO
         pass
 
-    def save(self, filename):
+    def save(self, pathname):
 
         s = self.schematic()
 
-        with open(filename, 'w') as fhandle:
+        with open(pathname, 'w') as fhandle:
             fhandle.write(s)
         self.dirty = False
 
@@ -409,53 +497,59 @@ class UIModelBase:
         s = '# Created by ' + self.ui.NAME + ' V' + self.ui.version + '\n'
 
         # Define node positions
-        foo = [str(node) for node in self.circuit.nodes.values()]
+        foo = [str(node) for node in self.circuit.nodes.values()
+               if not isnan(node.pos.x)]
+
         s += '; nodes={' + ', '.join(foo) + '}' + '\n'
 
         for cpt in self.circuit.elements.values():
-            s += str(cpt)
-            if cpt.gcpt is not None:
-                s += '; ' + cpt.gcpt.attr_string(self.STEP) + '\n'
+            s += str(cpt) + '\n'
 
         # FIXME, remove other preference string
         # Note, need a newline so string treated as a netlist string
         s += '; ' + self.preferences.schematic_preferences() + '\n'
         return s
 
-    def thing_create(self, cpt_type, x1, y1, x2, y2):
+    def thing_create(self, cpt_type, x1, y1, x2, y2, kind=''):
 
-        gcpt = cpt_make(cpt_type)
+        cpt_name = self.choose_cpt_name(cpt_type)
+        gcpt = cpt_make_from_type(cpt_type, cpt_name, kind=kind)
+        if gcpt is None:
+            return
 
-        cpt_name = self.choose_cpt_name(gcpt.type)
-        gcpt.name = cpt_name
-        net_parts = [cpt_name]
-
-        nodes = list(self.circuit.nodes)
+        all_node_names = list(self.circuit.nodes)
+        node_names = []
         positions = gcpt.assign_positions(x1, y1, x2, y2)
 
         for m, position in enumerate(positions):
+            if position is None:
+                continue
+
             node = self.circuit.nodes.by_position(position)
             if node is None:
-                node_name = self.choose_node_name(nodes)
-                nodes.append(node_name)
+                node_name = gcpt.choose_node_name(m, all_node_names)
+                all_node_names.append(node_name)
             else:
                 node_name = node.name
-            net_parts.append(node_name)
+            node_names.append(node_name)
 
-        net = ' '.join(net_parts)
+        netitem = gcpt.netitem(node_names, x1, y1, x2, y2, self.STEP)
 
         if self.ui.debug:
-            print('Adding ' + net)
+            print('Adding ' + netitem)
 
-        cct = self.circuit.add(net)
+        cct = self.circuit.add(netitem)
+        self.invalidate()
         cpt = cct[cpt_name]
 
         for m, position in enumerate(positions):
             cpt.nodes[m].pos = Pos(position)
 
+        attr_string = netitem.split(';', 1)[1]
+        gcpt.update(nodes=cpt.nodes, opts=Opts(attr_string))
+
         # Duck type
         cpt.gcpt = gcpt
-        gcpt.nodes = cpt.nodes
 
         self.cpt_draw(cpt)
 
@@ -543,7 +637,7 @@ class UIModelBase:
 
     def view(self):
 
-        cct = self.circuit
+        cct = Circuit(self.schematic())
         cct.draw()
 
     def voltage_annotate(self, cpt):
@@ -563,12 +657,16 @@ class UIModelBase:
 
     def node_draw(self, node):
 
+        if node.pos is None:
+            print('Pos unknown for ' + str(node))
+            return
+
         if node.port:
-            self.ui.layer.stroke_circle(
+            self.ui.sketcher.stroke_circle(
                 node.x, node.y, self.preferences.node_size,
                 color=self.preferences.node_color, alpha=1)
         else:
-            self.ui.layer.stroke_filled_circle(
+            self.ui.sketcher.stroke_filled_circle(
                 node.x, node.y, self.preferences.node_size,
                 color=self.preferences.node_color, alpha=1)
 
